/***************************************************************************
 *   Copyright (C) 2008 by Thomas Schildknecht <danakil.kde@gmail.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

// TODO better SVGs !
// TODO west, east, north, south tags
// TODO better minimalsize calculation
// TODO catch the drag n' drop from an horizontal panel to a vertical
//      (the elementName need to be updated)

#include "panelspacer.h"

#include <QPainter>
#include <QGraphicsLinearLayout>
#include <KConfigDialog>
#include <Plasma/Theme>
#include <Plasma/Containment>
#include <Plasma/Svg>
//#include <KDebug>

#define MINIMALSIZE 6    //--> TODO get ride of this (should be calculated based on the current SVG)

PanelSpacer::PanelSpacer(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
    , m_configureMode(false)
    , m_containment(0)
    , m_separatorSvg(0)
{
    resize(220, 80);
}

void PanelSpacer::init()
{
    KConfigGroup cg = config();
    m_spacerSize = cg.readEntry("spacerSize", MINIMALSIZE + 10);
    m_separatorStyle = (SeparatorStyle)cg.readEntry("separatorStyle", (int)Themed);
    m_separatorPosition = (SeparatorPosition)cg.readEntry("separatorPosition", (int)LeftOrTop);
    m_spacerStretch = cg.readEntry("separatorStretch", false);

    if (m_separatorStyle==Themed) {
        m_separatorElementId=cg.readEntry("separatorElementId", "1");
        initSvgSeparator(m_separatorElementId);
    }
}

 void PanelSpacer::initSvgSeparator(const QString &separatorId)
 {
     Plasma::FormFactor spacerFormFactor=formFactor();

     if (!m_separatorSvg) {
         m_separatorSvg = new Plasma::Svg(this);
         m_separatorSvg->setImagePath("widgets/panelspacer-separator");
         m_separatorSvg->setContainsMultipleImages(true);
         m_separatorSvg->resize(QSize());
    }

    // Set the stretch style by checking the existence of
    // specific elements
    if(m_separatorSvg->hasElement("separator-" + separatorId + "-stretchprop")) {
        m_separatorStretchType = StretchProportional;
    } else if(m_separatorSvg->hasElement("separator-" + separatorId + "-stretchheight")) {
        m_separatorStretchType = StretchHeight;
    } else {
        m_separatorStretchType = (SeparatorStretchType) 0;
    }

    // Determine the name of needed element and
    // calculate its ratio
    if (spacerFormFactor == Plasma::Horizontal) {
        m_separatorElementName="separator-";
        m_separatorElementName+=separatorId;
        m_separatorElementName+="-horizontal";
        if (m_separatorStretchType == StretchProportional) {
            m_separatorSvgRatio = ((qreal)m_separatorSvg->elementSize(m_separatorElementName).width() /
                                                        (qreal)m_separatorSvg->elementSize(m_separatorElementName).height());                                      
        }
     }
     else if (spacerFormFactor == Plasma::Vertical) {
        m_separatorElementName="separator-";
        m_separatorElementName+=separatorId;
        m_separatorElementName+="-vertical";
        if (m_separatorStretchType == StretchProportional) {
            m_separatorSvgRatio = ((qreal)m_separatorSvg->elementSize(m_separatorElementName).height() /
                                                        (qreal)m_separatorSvg->elementSize(m_separatorElementName).width());
        }
     }
}

void PanelSpacer::constraintsEvent(Plasma::Constraints constraints)
{
    Plasma::FormFactor spacerFormFactor=formFactor();
    QSizeF contentSize;
    int seppos;

    if (constraints & Plasma::FormFactorConstraint) {
        setHasConfigurationInterface(spacerFormFactor == Plasma::Horizontal ||
                                     spacerFormFactor == Plasma::Vertical);
        if (m_containment) {
            disconnect(m_containment, SIGNAL(appletAdded(Plasma::Applet*, const QPointF&)),
                       this, SLOT(calculateBlankSpace()));
            disconnect(m_containment, SIGNAL(appletRemoved(Plasma::Applet*)),
                       this, SLOT(calculateBlankSpace()));
        }
        m_containment = containment();
        if (m_containment) {
            connect(m_containment, SIGNAL(appletAdded(Plasma::Applet*, const QPointF&)),
                    this, SLOT(calculateBlankSpace()));
            connect(m_containment, SIGNAL(appletRemoved(Plasma::Applet*)),
                    this, SLOT(calculateBlankSpace()));
        }
    }

    if ((constraints & Plasma::SizeConstraint || constraints & Plasma::FormFactorConstraint)
            && m_separatorStyle == Themed ) {
       contentSize = size ();
       seppos=0;

       if (spacerFormFactor == Plasma::Horizontal) {
            // Calculate the width
            if(m_separatorStretchType == StretchProportional) {
                m_separatorWidth = m_separatorSvgRatio* contentSize.height();
            } else if (m_separatorStretchType == StretchHeight) {
                m_separatorWidth = m_separatorSvg->elementSize(m_separatorElementName).width();
            }
            // Calculate the alignement (X position)
            if (m_separatorPosition == Middle) {
                seppos = (contentSize.width() / 2) - (m_separatorWidth / 2);
            } else if (m_separatorPosition == RightOrBottom) {
                seppos = contentSize.width() - m_separatorWidth;
            }
            // Final size
            m_separatorRect.setRect(seppos, 0, m_separatorWidth, contentSize.height());
        }
        else if (spacerFormFactor == Plasma::Vertical) {
            // Calculate the height
            if (m_separatorStretchType == StretchProportional) {
                m_separatorWidth = m_separatorSvgRatio*contentSize.width();
            } else if (m_separatorStretchType == StretchHeight) {
                m_separatorWidth = m_separatorSvg->elementSize(m_separatorElementName).height();
            }
            // Calculate the alignement (Y position)
            if (m_separatorPosition == Middle) {
                seppos = (contentSize.height() / 2) - (m_separatorWidth / 2);
            } else if (m_separatorPosition == RightOrBottom) {
                seppos = contentSize.height() - m_separatorWidth;
            }
            // Final size
            m_separatorRect.setRect(0, seppos, contentSize.width(), m_separatorWidth);
        }
        calculateBlankSpace();
     }
}

void PanelSpacer::paintInterface(QPainter *p,
                                 const QStyleOptionGraphicsItem *option,
                                 const QRect &contentsRect)
{
    Q_UNUSED (option);

    QColor sepcol = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
    Plasma::FormFactor spacerFormFactor = formFactor();

    // warns the user if the plasmoid is on the desktop
    // and return
    if (spacerFormFactor == Plasma::Planar || spacerFormFactor == Plasma::MediaCenter) {
        p->setPen(QPen(sepcol));
        setHasConfigurationInterface(false);
        p->drawText(contentsRect, Qt::AlignLeft|Qt::AlignVCenter|Qt::TextWordWrap,
                i18n("I am a PanelSpacer so I am quite useless here... Please put me in a panel!"));
        return;
    }

    // draw a frame around the applet area when
    // the config dialog is displayed
    if (m_configureMode) {
        sepcol.setAlphaF(0.1);
        QBrush appletAreaB(sepcol);
        p->setPen(Qt::NoPen);
        p->setBrush(appletAreaB);

        if (spacerFormFactor == Plasma::Horizontal) {
            if (m_spacerSize >= MINIMALSIZE) {   // minimal size area
                p->drawRect(0, 0, m_spacerSize, contentsRect.bottom());
            }
            if (m_spacerStretch) {                              // stretched area
                if (m_spacerSize > MINIMALSIZE) {
                    p->drawRect(m_spacerSize + 4, contentsRect.top(),
                                contentsRect.right() - m_spacerSize - 4, contentsRect.bottom());
                } else {
                    p->drawRect(contentsRect);
                }
            }
        }
        else if (spacerFormFactor == Plasma::Vertical) {
            if (m_spacerSize >= MINIMALSIZE) {   // minimal size area
                p->drawRect(0, 0, contentsRect.right(), m_spacerSize);
            }
            if (m_spacerStretch) {                              // stretched area
                if (m_spacerSize > MINIMALSIZE) {
                    p->drawRect(0, m_spacerSize + 4, contentsRect.right(),
                                contentsRect.bottom()-m_spacerSize - 4);
                } else {
                    p->drawRect(contentsRect);
                }
            }
        }
    }

    // Paint the separator if Themed
    //
    if (m_separatorStyle == Themed) {
        m_separatorSvg->paint(p, m_separatorRect, m_separatorElementName);
    }
}

void PanelSpacer::createConfigurationInterface(KConfigDialog *parent)
{
    m_configureMode = true;

    QString elementOrientation;
    Plasma::Svg *svgTemp;
    QWidget *widget = new QWidget();
    uiConfig.setupUi(widget);
    parent->setButtons(KDialog::Ok);
    parent->addPage(widget, parent->windowTitle(), icon());

    uiConfig.horizontalSliderSize->setMinimum(MINIMALSIZE);
    uiConfig.horizontalSliderSize->setMaximum(m_maximumSize);
    uiConfig.horizontalSliderSize->setValue(m_spacerSize);
    uiConfig.checkBoxStretch->setChecked(m_spacerStretch);

    if (formFactor() == Plasma::Horizontal) {
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Left"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Right"));        
        elementOrientation="horizontal";
    }
    else if (formFactor() == Plasma::Vertical){
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Top"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Middle"));
        uiConfig.comboBoxSeparatorPosition->addItem(i18nc("Position of the separator", "Bottom"));        
        elementOrientation="vertical";   
    }

    // Init the style combobox
    // Load available styles from the svg file
    uiConfig.comboBoxSeparatorStyle->addItem(i18nc("Type of visual separator", "None"), "");
    svgTemp = new Plasma::Svg(this);
    svgTemp->setImagePath("widgets/panelspacer-separator");
    svgTemp->setContainsMultipleImages(true);
    svgTemp->resize(QSize());

    int i=1;  
    while (svgTemp->hasElement(QString("separator-" + QString::number(i) + "-" + elementOrientation))) {
        uiConfig.comboBoxSeparatorStyle->addItem(i18nc("Type of visual separator", "Style") + " " +
                QString::number(i), QString::number(i));
         i++;
    }
    delete svgTemp;

    uiConfig.comboBoxSeparatorStyle->setCurrentIndex(m_separatorElementId.toInt());
    uiConfig.comboBoxSeparatorPosition->setCurrentIndex(m_separatorPosition);
    if (m_separatorStyle == None) {
        uiConfig.comboBoxSeparatorPosition->setEnabled(false);
    }

    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(finished ()), this, SLOT(slotConfigClosed()));
    connect(uiConfig.checkBoxStretch, SIGNAL(toggled(bool)),
            this, SLOT(slotSpacerStretchChanged(bool)));
    connect(uiConfig.horizontalSliderSize, SIGNAL(sliderMoved(int)),
            this, SLOT(slotSpacerSizeChanged(int)));
    connect(uiConfig.comboBoxSeparatorStyle, SIGNAL(currentIndexChanged(int)),
            this, SLOT(slotSeparatorStyleChanged(int)));
    connect(uiConfig.comboBoxSeparatorPosition, SIGNAL(currentIndexChanged(int)),
            this, SLOT(slotSeparatorPositionChanged(int)));

    update();
}

void PanelSpacer::configAccepted()
{
    KConfigGroup cg = config();
    cg.writeEntry("spacerSize", uiConfig.horizontalSliderSize->value());
    cg.writeEntry("separatorPosition", uiConfig.comboBoxSeparatorPosition->currentIndex());
    cg.writeEntry("separatorStretch", uiConfig.checkBoxStretch->isChecked());
    cg.writeEntry("separatorElementId",  uiConfig.comboBoxSeparatorStyle->
                   itemData(uiConfig.comboBoxSeparatorStyle->currentIndex()).toString());

    if (uiConfig.comboBoxSeparatorStyle->currentIndex()==0) {
        cg.writeEntry("separatorStyle", (int)None);
    } else {
        cg.writeEntry("separatorStyle",  (int)Themed); 
    }

    emit configNeedsSaving();
}

void PanelSpacer::slotConfigClosed()
{
    m_configureMode = false;
    update();
}

void PanelSpacer::slotSpacerStretchChanged(bool checkstate)
{
    m_spacerStretch = checkstate;
    updateConstraints();
}

void PanelSpacer::slotSeparatorPositionChanged(int index)
{
    m_separatorPosition = (SeparatorPosition)index;
    updateConstraints();
    update();
}

void PanelSpacer::slotSeparatorStyleChanged(int index)
{
    if (uiConfig.comboBoxSeparatorStyle->currentIndex() == 0) {
        m_separatorStyle = None;
        uiConfig.comboBoxSeparatorPosition->setEnabled(false);
    }
    else {
        m_separatorStyle = Themed;
        m_separatorElementId = QString::number(index);
        uiConfig.comboBoxSeparatorPosition->setEnabled(true);
        initSvgSeparator(m_separatorElementId);
    }

    updateConstraints();
    update();
}

void PanelSpacer::slotSpacerSizeChanged(int size)
{
    m_spacerSize = size;
    updateConstraints();
    update();
}

void PanelSpacer::calculateBlankSpace()
{
    Plasma::FormFactor spacerFormFactor=formFactor();
    m_maximumSize = 600;

    if (m_containment) {
        if (spacerFormFactor == Plasma::Horizontal) {
            m_maximumSize = m_containment->geometry().width();
        } else {
            m_maximumSize = m_containment->geometry().height();
        }
        qreal spacing = 0;
        QGraphicsLinearLayout *lay = dynamic_cast<QGraphicsLinearLayout*>(m_containment->layout());
        if (lay) {
            spacing = lay->spacing();
        }
        foreach (Plasma::Applet* applet, m_containment->applets()) {
            if (applet != this) {
                kDebug() << applet->name() << applet->preferredSize() << spacing;
                if (spacerFormFactor == Plasma::Horizontal) {
                    m_maximumSize -= applet->preferredWidth() + spacing;
                } else {
                    m_maximumSize -= applet->preferredHeight() + spacing;
                }
            }
        }
    }
    m_spacerSize = qMin(m_spacerSize, m_maximumSize);
    if (spacerFormFactor == Plasma::Horizontal) {
        setMinimumWidth(m_spacerSize);
        if (!m_spacerStretch) {
            setMaximumWidth(m_spacerSize);
        } else {
            setMaximumWidth(-1);
        }
    }
    else if (spacerFormFactor == Plasma::Vertical) {
        setMinimumHeight(m_spacerSize);
        if (!m_spacerStretch) {
            setMaximumHeight(m_spacerSize);
        } else {
            setMaximumHeight(-1);
        }
    }
}

#include "panelspacer.moc"
