/***************************************************************************
 *   Copyright (C) 2008 by Thomas Schildknecht <danakil.kde@gmail.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef PLASMAPANELSPACER_HEADER
#define PLASMAPANELSPACER_HEADER

#include <Plasma/Applet>
#include "ui_panelspacerConfig.h"

namespace Plasma {
    class Svg;
    class Containment;
}

class PanelSpacer : public Plasma::Applet
{
    Q_OBJECT
    public:
        enum SeparatorStyle {
            None = 0,
            Themed = 1
        };
        enum SeparatorPosition {
            LeftOrTop = 0,
            Middle = 1,
            RightOrBottom = 2
        };
        enum SeparatorStretchType {
            StretchProportional = 0,
            StretchHeight = 1
        };

        PanelSpacer(QObject *parent, const QVariantList &args);

        void init();
        void paintInterface(QPainter *painter,
                            const QStyleOptionGraphicsItem *option,
                            const QRect& contentsRect);

    protected :
        void createConfigurationInterface(KConfigDialog *parent);
        void initSvgSeparator(const QString &separatorId);

    public slots :
        void slotSpacerStretchChanged(bool checkstate);
        void slotSpacerSizeChanged(int size);
        void slotSeparatorPositionChanged(int index);
        void slotSeparatorStyleChanged(int index);
        void slotConfigClosed();

    protected slots:
        void configAccepted();
        void constraintsEvent(Plasma::Constraints constraints);
        void calculateBlankSpace();

    private:
        Ui::panelspacerConfig uiConfig;

        bool m_spacerStretch;
        bool m_configureMode;

        qreal m_spacerSize;
        qreal m_maximumSize;
        qreal m_separatorSvgRatio;
        qreal m_separatorWidth;     //--> = Width for horizontal panel,
                                    //    = Height for vertical panel

        SeparatorPosition m_separatorPosition;
        SeparatorStyle m_separatorStyle;
        SeparatorStretchType m_separatorStretchType;
        Plasma::Containment *m_containment;
        Plasma::Svg *m_separatorSvg;
        QString m_separatorElementName;
        QRectF m_separatorRect;
        QString m_separatorElementId;
};

K_EXPORT_PLASMA_APPLET(panelspacer, PanelSpacer)

#endif
